==> imaze1.0.tar.gz: imaze/source/rechne_karte.c <==
/*
**                      Softwarepraktikum iMaze
**                              1993/94
**                Joerg Czeranski    Hans-Ulrich Kiel
**
** Datei: rechne_karte.c
**
** Kommentar:
**  Berechnet aus den Labyrinth-Daten den aktuellen Inhalt der Karte
**  und legt die zu zeichnenden Grafik-Objekte in den Datenstrukturen
**  ab (in Bildpunkten)
*/


#include "global.h"
#include "speicher.h"
#include "labyrinth.h"
#include "farben.h"
#include "spieler.h"
#include "grafik.h"
#include "rechne.h"

static char sccsid[] = "@(#)rechne_karte.c	1.8 5/17/94";


/*
** skalier_x
**  skaliert Labyrinth-x-Koordinaten im feinen Raster auf Bildpunkte
**
** Parameter:
**  x: x-Koordinate im feinen Raster
**
** Rueckgabewert:
**  x-Koordinate in Bildpunkten (0 ... FENSTER_BREITE)
*/
static int skalier_x(x)
int x;
{
	return x * (FENSTER_BREITE / RASPT) / feldbreite;
}


/*
** skalier_y
**  skaliert Labyrinth-y-Koordinaten im feinen Raster auf Bildpunkte
**
** Parameter:
**  y: y-Koordinate im feinen Raster
**
** Rueckgabewert:
**  y-Koordinate in Bildpunkten (0 ... FENSTER_HOEHE)
*/
static int skalier_y(y)
int y;
{
	return y * (FENSTER_HOEHE / RASPT) / feldlaenge;
}


/*
** rechnelinie
**  skaliert die Koordinaten zweier aneinanderliegender Waende,
**  bestimmt die jeweiligen Typen (siehe grafik.h) und traegt das
**  berechnete Linienpaar in die Linien-Liste ein
**
** Parameter:
**  anzlinien: Zeiger auf die Anzahl der zu zeichnenden Linienpaare
**  linien: Zeiger auf die Liste der zu zeichnenden Linienpaare
**  x: x-Koordinate des linken/oberen Wandendes im groben Raster
**  y: y-Koordinate des linken/oberen Wandendes im groben Raster
**  senkrecht: Flag, ob das Linienpaar senkrecht verlaeuft
**  wand1: Zeiger auf die eine Wand
**  wand2: Zeiger auf die andere Wand
**
** Seiteneffekte:
**  *anzlinien wird erhoeht, *linien um ein Element verlaengert
*/
static void rechnelinie(anzlinien, linien, x, y, senkrecht, wand1, wand2)
int *anzlinien, x, y;
struct kartenlinie **linien;
u_char senkrecht;
struct wand *wand1, *wand2;
{
	struct kartenlinie *linie; /* Zeiger auf das neue
	                              Linienpaar in *linien */

	/* falls beide Waende transparent sind, kein Linienpaar eintragen */
	if (wand1->farbe == TRANSPARENT && wand2->farbe == TRANSPARENT)
		return;

	liste_verlaengern(anzlinien, (void **)linien,
		sizeof(struct kartenlinie));
	linie = &(*linien)[*anzlinien - 1];

	/* Koordinaten skalieren */
	linie->ecke.x = skalier_x(x * RASPT);
	linie->ecke.y = skalier_y(y * RASPT);
	linie->laenge = senkrecht ?
		skalier_y((y + 1) * RASPT) - skalier_y(y * RASPT) :
		skalier_x((x + 1) * RASPT) - skalier_x(x * RASPT);
	linie->senkrecht = senkrecht;

	/* Tueren haben Typ TUERLINIE, Waende Typ WANDLINIE, transparente
	   Waende Typ KEINELINIE */
	linie->typ1 = wand1->tuer ? TUERLINIE : wand1->farbe == TRANSPARENT ?
		KEINELINIE : WANDLINIE;
	linie->typ2 = wand2->tuer ? TUERLINIE : wand2->farbe == TRANSPARENT ?
		KEINELINIE : WANDLINIE;
}


/* bis hier lokaler Teil                       */
/***********************************************/
/* ab hier globaler Teil                       */


/*
** rechne_karte_linien
**  berechnet alle auf der Karte zu zeichnenden Linienpaare
**
** Parameter:
**  anzlinien: Zeiger auf die Anzahl der zu zeichnenden Linienpaare
**  linien: Zeiger auf die Liste der zu zeichnenden Linienpaare
**
** Seiteneffekte:
**  *anzlinien und *linien werden gesetzt
*/
void rechne_karte_linien(anzlinien, linien)
int *anzlinien;
struct kartenlinie **linien;
{
	int x, y; /* x/y-Koordinaten im groben Raster */

	/* berechnet alle Nord- und Suedwaende */
	for (x = 0; x < feldbreite; x++)
	{
		/* die noerdlichste Wand ist einzeln und wird deshalb zweimal
		   uebergeben */
		rechnelinie(anzlinien, linien, x, 0, 0,
			&spielfeld[x][0][NORD],
			&spielfeld[x][0][NORD]);

		/* die uebrigen Nord- und Suedwaende sind jeweils Paare */
		for (y = 1; y < feldlaenge; y++)
			rechnelinie(anzlinien, linien, x, y, 0,
				&spielfeld[x][y - 1][SUED],
				&spielfeld[x][y][NORD]);

		/* die suedlichste Wand ist einzeln und wird deshalb zweimal
		   uebergeben */
		rechnelinie(anzlinien, linien, x, feldlaenge, 0,
			&spielfeld[x][feldlaenge - 1][SUED],
			&spielfeld[x][feldlaenge - 1][SUED]);
	}

	/* berechnet alle West- und Ostwaende */
	for (y = 0; y < feldlaenge; y++)
	{
		/* die westlichste Wand ist einzeln und wird deshalb zweimal
		   uebergeben */
		rechnelinie(anzlinien, linien, 0, y, 1,
			&spielfeld[0][y][WEST],
			&spielfeld[0][y][WEST]);

		/* die uebrigen West- und Ostwaende sind jeweils Paare */
		for (x = 1; x < feldbreite; x++)
			rechnelinie(anzlinien, linien, x, y, 1,
				&spielfeld[x - 1][y][OST],
				&spielfeld[x][y][WEST]);

		/* die oestlichste Wand ist einzeln und wird deshalb zweimal
		   uebergeben */
		rechnelinie(anzlinien, linien, feldbreite, y, 1,
			&spielfeld[feldbreite - 1][y][OST],
			&spielfeld[feldbreite - 1][y][OST]);
	}
}


/*
** rechne_karte_kreise
**  berechnet alle auf der Karte zu zeichnenden Kreise (Spieler)
**
** Parameter:
**  anzkreise: Zeiger auf die Anzahl der zu zeichnenden Kreise
**  kreise: Zeiger auf die Liste der zu zeichnenden Kreise
**  anzspieler: Anzahl der Spieler
**  spieler: Liste der Spieler
**
** Seiteneffekte:
**  *anzkreise und *kreise werden gesetzt
*/
void rechne_karte_kreise(anzkreise, kreise, anzspieler, spieler)
int *anzkreise, anzspieler;
struct kartenkreis **kreise;
struct spieler *spieler;
{
	int i; /* Index fuer Spieler */

	for (i = 0; i < anzspieler; i++)
	{
		liste_verlaengern(anzkreise, (void **)kreise,
			sizeof(struct kartenkreis));

		/* Koordinaten des Spielers skalieren */
		(*kreise)[*anzkreise - 1].mittelpunkt.x =
			skalier_x(XPOS(spieler[i].pos));
		(*kreise)[*anzkreise - 1].mittelpunkt.y =
			skalier_y(YPOS(spieler[i].pos));

		/* Radius des Spielers skalieren */
		(*kreise)[*anzkreise - 1].radiusx = skalier_x(KUGELRAD);
		(*kreise)[*anzkreise - 1].radiusy = skalier_y(KUGELRAD);

		/* Farbe direkt kopieren */
		(*kreise)[*anzkreise - 1].farbe = spieler[i].farbe;
	}
}

==> imaze1.2.tar.gz: imaze/source/rechne_karte.c <==
/*
**                      Softwarepraktikum iMaze
**                              1993/94
**                Joerg Czeranski    Hans-Ulrich Kiel
**
** Datei: rechne_karte.c
**
** Kommentar:
**  Berechnet aus den Labyrinth-Daten den aktuellen Inhalt der Karte
**  und legt die zu zeichnenden Grafik-Objekte in den Datenstrukturen
**  ab (in Bildpunkten)
*/


#include "global.h"
#include "speicher.h"
#include "labyrinth.h"
#include "farben.h"
#include "spieler.h"
#include "grafik.h"
#include "rechne.h"

static char sccsid[] = "@(#)rechne_karte.c	1.8 17 May 1994";


/*
** skalier_x
**  skaliert Labyrinth-x-Koordinaten im feinen Raster auf Bildpunkte
**
** Parameter:
**  x: x-Koordinate im feinen Raster
**
** Rueckgabewert:
**  x-Koordinate in Bildpunkten (0 ... FENSTER_BREITE)
*/
static int skalier_x(x)
int x;
{
	return x * (FENSTER_BREITE / RASPT) / feldbreite;
}


/*
** skalier_y
**  skaliert Labyrinth-y-Koordinaten im feinen Raster auf Bildpunkte
**
** Parameter:
**  y: y-Koordinate im feinen Raster
**
** Rueckgabewert:
**  y-Koordinate in Bildpunkten (0 ... FENSTER_HOEHE)
*/
static int skalier_y(y)
int y;
{
	return y * (FENSTER_HOEHE / RASPT) / feldlaenge;
}


/*
** rechnelinie
**  skaliert die Koordinaten zweier aneinanderliegender Waende,
**  bestimmt die jeweiligen Typen (siehe grafik.h) und traegt das
**  berechnete Linienpaar in die Linien-Liste ein
**
** Parameter:
**  anzlinien: Zeiger auf die Anzahl der zu zeichnenden Linienpaare
**  linien: Zeiger auf die Liste der zu zeichnenden Linienpaare
**  x: x-Koordinate des linken/oberen Wandendes im groben Raster
**  y: y-Koordinate des linken/oberen Wandendes im groben Raster
**  senkrecht: Flag, ob das Linienpaar senkrecht verlaeuft
**  wand1: Zeiger auf die eine Wand
**  wand2: Zeiger auf die andere Wand
**
** Seiteneffekte:
**  *anzlinien wird erhoeht, *linien um ein Element verlaengert
*/
static void rechnelinie(anzlinien, linien, x, y, senkrecht, wand1, wand2)
int *anzlinien, x, y;
struct kartenlinie **linien;
u_char senkrecht;
struct wand *wand1, *wand2;
{
	struct kartenlinie *linie; /* Zeiger auf das neue
	                              Linienpaar in *linien */

	/* falls beide Waende transparent sind, kein Linienpaar eintragen */
	if (wand1->farbe == TRANSPARENT && wand2->farbe == TRANSPARENT)
		return;

	liste_verlaengern(anzlinien, (void **)linien,
		sizeof(struct kartenlinie));
	linie = &(*linien)[*anzlinien - 1];

	/* Koordinaten skalieren */
	linie->ecke.x = skalier_x(x * RASPT);
	linie->ecke.y = skalier_y(y * RASPT);
	linie->laenge = senkrecht ?
		skalier_y((y + 1) * RASPT) - skalier_y(y * RASPT) :
		skalier_x((x + 1) * RASPT) - skalier_x(x * RASPT);
	linie->senkrecht = senkrecht;

	/* Tueren haben Typ TUERLINIE, Waende Typ WANDLINIE, transparente
	   Waende Typ KEINELINIE */
	linie->typ1 = wand1->tuer ? TUERLINIE : wand1->farbe == TRANSPARENT ?
		KEINELINIE : WANDLINIE;
	linie->typ2 = wand2->tuer ? TUERLINIE : wand2->farbe == TRANSPARENT ?
		KEINELINIE : WANDLINIE;
}


/* bis hier lokaler Teil                       */
/***********************************************/
/* ab hier globaler Teil                       */


/*
** rechne_karte_linien
**  berechnet alle auf der Karte zu zeichnenden Linienpaare
**
** Parameter:
**  anzlinien: Zeiger auf die Anzahl der zu zeichnenden Linienpaare
**  linien: Zeiger auf die Liste der zu zeichnenden Linienpaare
**
** Seiteneffekte:
**  *anzlinien und *linien werden gesetzt
*/
void rechne_karte_linien(anzlinien, linien)
int *anzlinien;
struct kartenlinie **linien;
{
	int x, y; /* x/y-Koordinaten im groben Raster */

	/* berechnet alle Nord- und Suedwaende */
	for (x = 0; x < feldbreite; x++)
	{
		/* die noerdlichste Wand ist einzeln und wird deshalb zweimal
		   uebergeben */
		rechnelinie(anzlinien, linien, x, 0, 0,
			&spielfeld[x][0][NORD],
			&spielfeld[x][0][NORD]);

		/* die uebrigen Nord- und Suedwaende sind jeweils Paare */
		for (y = 1; y < feldlaenge; y++)
			rechnelinie(anzlinien, linien, x, y, 0,
				&spielfeld[x][y - 1][SUED],
				&spielfeld[x][y][NORD]);

		/* die suedlichste Wand ist einzeln und wird deshalb zweimal
		   uebergeben */
		rechnelinie(anzlinien, linien, x, feldlaenge, 0,
			&spielfeld[x][feldlaenge - 1][SUED],
			&spielfeld[x][feldlaenge - 1][SUED]);
	}

	/* berechnet alle West- und Ostwaende */
	for (y = 0; y < feldlaenge; y++)
	{
		/* die westlichste Wand ist einzeln und wird deshalb zweimal
		   uebergeben */
		rechnelinie(anzlinien, linien, 0, y, 1,
			&spielfeld[0][y][WEST],
			&spielfeld[0][y][WEST]);

		/* die uebrigen West- und Ostwaende sind jeweils Paare */
		for (x = 1; x < feldbreite; x++)
			rechnelinie(anzlinien, linien, x, y, 1,
				&spielfeld[x - 1][y][OST],
				&spielfeld[x][y][WEST]);

		/* die oestlichste Wand ist einzeln und wird deshalb zweimal
		   uebergeben */
		rechnelinie(anzlinien, linien, feldbreite, y, 1,
			&spielfeld[feldbreite - 1][y][OST],
			&spielfeld[feldbreite - 1][y][OST]);
	}
}


/*
** rechne_karte_kreise
**  berechnet alle auf der Karte zu zeichnenden Kreise (Spieler)
**
** Parameter:
**  anzkreise: Zeiger auf die Anzahl der zu zeichnenden Kreise
**  kreise: Zeiger auf die Liste der zu zeichnenden Kreise
**  anzspieler: Anzahl der Spieler
**  spieler: Liste der Spieler
**
** Seiteneffekte:
**  *anzkreise und *kreise werden gesetzt
*/
void rechne_karte_kreise(anzkreise, kreise, anzspieler, spieler)
int *anzkreise, anzspieler;
struct kartenkreis **kreise;
struct spieler *spieler;
{
	int i; /* Index fuer Spieler */

	for (i = 0; i < anzspieler; i++)
	{
		liste_verlaengern(anzkreise, (void **)kreise,
			sizeof(struct kartenkreis));

		/* Koordinaten des Spielers skalieren */
		(*kreise)[*anzkreise - 1].mittelpunkt.x =
			skalier_x(XPOS(spieler[i].pos));
		(*kreise)[*anzkreise - 1].mittelpunkt.y =
			skalier_y(YPOS(spieler[i].pos));

		/* Radius des Spielers skalieren */
		(*kreise)[*anzkreise - 1].radiusx = skalier_x(KUGELRAD);
		(*kreise)[*anzkreise - 1].radiusy = skalier_y(KUGELRAD);

		/* Farbe direkt kopieren */
		(*kreise)[*anzkreise - 1].farbe = spieler[i].farbe;
	}
}
